<?php

namespace Drupal\php_password\Password;

use Drupal\Core\Password\PasswordInterface;

/**
 * Proxy class to coordinate between legacy password hashes and native hashes.
 */
class Drupal8Password implements PasswordInterface {

  /**
   * The PHP password hashing service.
   *
   * @see \Drupal\php_password\Password\PHPPassword
   */
  protected PasswordInterface $phpPassword;

  /**
   * The Drupal 7 password hashing service.
   *
   * @see \Drupal\Core\Password\PhpassHashedPassword
   */
  protected PasswordInterface $drupal7Password;

  /**
   * Constructs a new password hashing instance.
   *
   * @param \Drupal\Core\Password\PasswordInterface $php_password
   *   The PHP password hashing service.
   * @param \Drupal\Core\Password\PasswordInterface $drupal7_password
   *   The Drupal7 password hashing service.
   */
  public function __construct(PasswordInterface $php_password, PasswordInterface $drupal7_password) {
    $this->phpPassword = $php_password;
    $this->drupal7Password = $drupal7_password;
  }

  /**
   * {@inheritdoc}
   */
  public function hash($password): string {
    return $this->phpPassword->hash($password);
  }

  /**
   * {@inheritdoc}
   */
  public function check($password, $hash): bool {

    // MD5 migrated password (Drupal 6).
    $prefix = substr($hash, 0, 2);
    if ($prefix === 'U$') {
      $hash = substr($hash, 1);
      $prefix = substr($hash, 0, 2);
      $password = md5($password);
    }

    return match ($prefix) {
      '$S', '$H', '$P' => $this->drupal7Password->check($password, $hash),
      default => $this->phpPassword->check($password, $hash),
    };
  }

  /**
   * {@inheritdoc}
   */
  public function needsRehash($hash): bool {
    return $this->phpPassword->needsRehash($hash);
  }

}
