<?php

namespace Drupal\Tests\php_password\Unit\Password;

use Drupal\php_password\Password\PhpPassword2;
use Drupal\Tests\UnitTestCase;

/**
 * Test PhpPasswordTest native password implementation.
 *
 * @coversDefaultClass \Drupal\php_password\Password\PhpPassword2
 * @group php_password
 */
class PhpPassword2Test extends UnitTestCase {

  /**
   * @covers ::hash
   * @dataProvider providePasswordOptions
   */
  public function testHash($algo, $options): void {
    $password_implementation = new PhpPassword2($algo, $options);
    $hash = $password_implementation->hash('mock_password');
    // Each hash is unique with a salt, so we verify it validates.
    $this->assertTrue(password_verify('mock_password', $hash));
    $info = password_get_info($hash);
    $this->assertEquals($algo, $info['algo']);
    $this->assertEquals($options, $info['options']);
  }

  /**
   * @covers ::hash
   * @covers ::getOptions
   */
  public function testHashDefault(): void {
    $password_implementation = new PhpPassword2();
    $hash = $password_implementation->hash('mock_password');
    // Each hash is unique with a salt, so we verify it validates.
    $this->assertTrue(password_verify('mock_password', $hash));
    $info = password_get_info($hash);
    $this->assertEquals(PASSWORD_DEFAULT, $info['algo']);
    $this->assertEquals(['cost' => PASSWORD_BCRYPT_DEFAULT_COST],
      $info['options']);

    $password_implementation = new PhpPassword2(PASSWORD_ARGON2I);
    $hash = $password_implementation->hash('mock_password');
    // Each hash is unique with a salt, so we verify it validates.
    $this->assertTrue(password_verify('mock_password', $hash));
    $info = password_get_info($hash);
    $this->assertEquals(PASSWORD_ARGON2I, $info['algo']);
    $this->assertEquals([
      'memory_cost' => PASSWORD_ARGON2_DEFAULT_MEMORY_COST,
      'time_cost' => PASSWORD_ARGON2_DEFAULT_TIME_COST,
      'threads' => PASSWORD_ARGON2_DEFAULT_THREADS,
    ], $info['options']);
  }

  /**
   * @covers ::needsRehash
   */
  public function testNeedsRehash(): void {
    $this->markTestIncomplete('needs testing.');
  }

  /**
   * @covers ::check
   */
  public function testCheck(): void {
    $this->markTestIncomplete('needs testing.');
  }

  /**
   * Data provider for password hashing options.
   */
  public function providePasswordOptions(): array {
    return [
      [PASSWORD_DEFAULT, ['cost' => 5]],
      [PASSWORD_DEFAULT, ['cost' => 10]],
      [
        PASSWORD_ARGON2I,
        [
          'memory_cost' => 1024,
          'time_cost' => 2,
          'threads' => 2,
        ],
      ],
      [
        PASSWORD_ARGON2ID,
        [
          'memory_cost' => 1024,
          'time_cost' => 2,
          'threads' => 2,
        ],
      ],
    ];
  }

}
