<?php

namespace Drupal\Tests\php_password\Unit\Password;

use Drupal\php_password\Password\PhpPassword;
use Drupal\Tests\UnitTestCase;

/**
 * Test PhpPasswordTest native password implementation.
 *
 * @coversDefaultClass \Drupal\php_password\Password\PhpPassword
 * @group php_password
 */
class PhpPasswordTest extends UnitTestCase {

  /**
   * @covers ::hash
   * @dataProvider providePasswordOptions
   */
  public function testHash($cost, $algo): void {
    $password_implementation = new PhpPassword($cost, $algo);
    $hash = $password_implementation->hash('mock_password');
    // Each hash is unique with a salt so we verify it validates.
    $this->assertTrue(password_verify('mock_password', $hash));
    $info = password_get_info($hash);
    $this->assertEquals($algo, $info['algo']);
    $this->assertEquals(['cost' => $cost], $info['options']);
  }

  /**
   * @covers ::hash
   * @covers ::getOptions
   * @dataProvider providePasswordOptions
   */
  public function testHashDefault($cost): void {
    $password_implementation = new PhpPassword($cost);
    $hash = $password_implementation->hash('mock_password');
    // Each hash is unique with a salt so we verify it validates.
    $this->assertTrue(password_verify('mock_password', $hash));
    $info = password_get_info($hash);
    $this->assertEquals(PASSWORD_DEFAULT, $info['algo']);
    $this->assertEquals(['cost' => $cost], $info['options']);
  }

  /**
   * @covers ::needsRehash
   */
  public function testNeedsRehash(): void {
    $this->markTestIncomplete('needs testing.');
  }

  /**
   * @covers ::check
   */
  public function testCheck(): void {
    $this->markTestIncomplete('needs testing.');
  }

  /**
   * Data provider for password hashing options.
   */
  public function providePasswordOptions(): array {
    return [
      [5, PASSWORD_DEFAULT],
      [10, PASSWORD_DEFAULT],
    ];
  }

}
